@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Base;
 
-our $DATE = '2016-01-22'; # DATE
-our $VERSION = '1.48'; # VERSION
+our $DATE = '2016-01-23'; # DATE
+our $VERSION = '1.49'; # VERSION
 
 use 5.010001;
 use strict;
@@ -593,7 +593,7 @@ sub _read_config {
 
     my ($self, $r) = @_;
 
-    $log->tracef("[pericmd] Finding config files ...");
+    #$log->tracef("[pericmd] Finding config files ...");
     my $res = Perinci::CmdLine::Util::Config::read_config(
         config_paths     => $r->{config_paths},
         config_filename  => $self->config_filename,
@@ -603,8 +603,9 @@ sub _read_config {
     die $res unless $res->[0] == 200;
     $r->{config} = $res->[2];
     $r->{read_config_files} = $res->[3]{'func.read_files'};
-    $log->tracef("[pericmd] Read config files: %s",
-                 $r->{'read_config_files'});
+    $r->{_config_section_read_order} = $res->[3]{'func.section_read_order'}; # we currently don't want to publish this request key
+    #$log->tracef("[pericmd] Read config files: %s",
+    #             $r->{'read_config_files'});
 }
 
 sub __min(@) {
@@ -1415,7 +1416,7 @@ Perinci::CmdLine::Base - Base class for Perinci::CmdLine{::Classic,::Lite}
 
 =head1 VERSION
 
-This document describes version 1.48 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2016-01-22.
+This document describes version 1.49 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2016-01-23.
 
 =head1 DESCRIPTION
 
@@ -2224,7 +2225,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Cm
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-CmdLine-Lite>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-CmdLine-Lite>.
 
 =head1 BUGS
 
