@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Util::Config;
 
-our $DATE = '2016-01-22'; # DATE
-our $VERSION = '1.48'; # VERSION
+our $DATE = '2016-01-23'; # DATE
+our $VERSION = '1.49'; # VERSION
 
 use 5.010;
 use strict;
@@ -61,8 +61,19 @@ sub read_config {
     my $reader = Config::IOD::Reader->new;
     my %res;
     my @read;
+    my %section_read_order;
     for my $path (@$paths) {
-        my $hoh = $reader->read_file($path);
+        $log->tracef("[pericmd] Reading config file '%s' ...", $path);
+        my $hoh = $reader->read_file(
+            $path,
+            sub {
+                my %args = @_;
+                return unless $args{event} eq 'section';
+                my $section = $args{section};
+                return if $section_read_order{$section};
+                $section_read_order{$section} = 1 + keys %section_read_order;
+            },
+        );
         push @read, $path;
         for my $section (keys %$hoh) {
             my $hash = $hoh->{$section};
@@ -71,7 +82,10 @@ sub read_config {
             }
         }
     }
-    [200, "OK", \%res, {'func.read_files' => \@read}];
+    [200, "OK", \%res, {
+        'func.read_files' => \@read,
+        'func.section_read_order' => \%section_read_order,
+    }];
 }
 
 $SPEC{get_args_from_config} = {
@@ -105,17 +119,20 @@ sub get_args_from_config {
         $meta = Perinci::Sub::Normalize::normalize_function_metadata($meta);
     }
 
-    # put GLOBAL before all other sections
+    my $csro = $r->{_config_section_read_order} // {};
     my @sections = sort {
+        # put GLOBAL before all other sections
         ($a eq 'GLOBAL' ? 0:1) <=> ($b eq 'GLOBAL' ? 0:1) ||
-            $a cmp $b
+            # sort according to the order the section is seen in the file
+            ($csro->{$a} // 0) <=> ($csro->{$b} // 0) ||
+                $a cmp $b
         } keys %$conf;
 
     my %seen_profiles; # for debugging message
     for my $section (@sections) {
         my %keyvals;
         for my $word (split /\s+/, ($section eq 'GLOBAL' ? '' : $section)) {
-            if ($word =~ /(.+)=(.*)/) {
+            if ($word =~ /(.+?)=(.*)/) {
                 $keyvals{$1} = $2;
             } else {
                 # old syntax, will be removed sometime in the future
@@ -130,25 +147,98 @@ sub get_args_from_config {
         # if there is a subcommand name, use section with no subcommand=... or
         # the matching subcommand
         if (length $scn) {
-            next if length($sect_scn) && $sect_scn ne $scn;
+            if (length($sect_scn) && $sect_scn ne $scn) {
+                $log->tracef(
+                    "[pericmd] Skipped config section '%s' (%s)",
+                    $section, "subcommand does not match '$scn'",
+                );
+                next;
+            }
         } else {
-            next if length $sect_scn;
+            if (length $sect_scn) {
+                $log->tracef(
+                    "[pericmd] Skipped config section '%s' (%s)",
+                    $section, "only for a certain subcommand",
+                );
+                next;
+            }
         }
 
         # if user chooses a profile, only use section with no profile=... or the
         # matching profile
         if (defined $profile) {
-            next if defined($sect_profile) && $sect_profile ne $profile;
+            if (defined($sect_profile) && $sect_profile ne $profile) {
+                $log->tracef(
+                    "[pericmd] Skipped config section '%s' (%s)",
+                    $section, "profile does not match '$profile'",
+                );
+                next;
+            }
             $found = 1 if defined($sect_profile) && $sect_profile eq $profile;
         } else {
-            next if defined($sect_profile);
+            if (defined($sect_profile)) {
+                $log->tracef(
+                    "[pericmd] Skipped config section '%s' (%s)",
+                    $section, "only for a certain profile",
+                );
+                next;
+            }
         }
 
         # only use section marked with program=... if the program name matches
         if (defined($progn) && defined($keyvals{program})) {
-            next unless $progn eq $keyvals{program};
+            if ($progn ne $keyvals{program}) {
+                $log->tracef(
+                    "[pericmd] Skipped config section '%s' (%s)",
+                    $section, "program does not match '$progn'",
+                );
+                next;
+            }
         }
 
+        # if user specifies env=... then apply filtering by ENV variable
+        if (defined(my $env = $keyvals{env})) {
+            my ($var, $val);
+            if (($var, $val) = $env =~ /\A(\w+)=(.*)\z/) {
+                if (($ENV{$var} // '') ne $val) {
+                    $log->tracef(
+                        "[pericmd] Skipped config section '%s' (%s)",
+                        $section, "env $var has non-matching value '".
+                            ($ENV{$var} // '')."'",
+                    );
+                    next;
+                }
+            } elsif (($var, $val) = $env =~ /\A(\w+)!=(.*)\z/) {
+                if (($ENV{$var} // '') eq $val) {
+                    $log->tracef(
+                        "[pericmd] Skipped config section '%s' (%s)",
+                        $section, "env $var has that value",
+                    );
+                    next;
+                }
+            } elsif (($var, $val) = $env =~ /\A(\w+)\*=(.*)\z/) {
+                if (index(($ENV{$var} // ''), $val) < 0) {
+                    $log->tracef(
+                        "[pericmd] Skipped config section '%s' (%s)",
+                        $section, "env $var has value '".
+                            ($ENV{$var} // '')."' which does not contain the ".
+                                "requested string"
+                    );
+                    next;
+                }
+            } else {
+                if (!$ENV{$env}) {
+                    $log->tracef(
+                        "[pericmd] Skipped config section '%s' (%s)",
+                        $section, "env $env is not set/true",
+                    );
+                    next;
+                }
+            }
+        }
+
+        $log->tracef("[pericmd] Reading config section '%s'", $section);
+
         my $as = $meta->{args} // {};
         for my $k (keys %{ $conf->{$section} }) {
             my $v = $conf->{$section}{$k};
@@ -183,8 +273,8 @@ sub get_args_from_config {
             }
         }
     }
-    $log->tracef("[pericmd] Seen config profiles: %s",
-                 [sort keys %seen_profiles]);
+    #$log->tracef("[pericmd] Seen config profiles: %s",
+    #             [sort keys %seen_profiles]);
 
     [200, "OK", $args, {'func.found'=>$found}];
 }
@@ -204,7 +294,7 @@ Perinci::CmdLine::Util::Config - Utility routines related to config files
 
 =head1 VERSION
 
-This document describes version 1.48 of Perinci::CmdLine::Util::Config (from Perl distribution Perinci-CmdLine-Lite), released on 2016-01-22.
+This document describes version 1.49 of Perinci::CmdLine::Util::Config (from Perl distribution Perinci-CmdLine-Lite), released on 2016-01-23.
 
 =head1 FUNCTIONS
 
@@ -300,7 +390,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Cm
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-CmdLine-Lite>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-CmdLine-Lite>.
 
 =head1 BUGS
 
